package com.yunan.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.yunan.dto.LoginDTO;
import com.yunan.dto.RegisterDTO;
import com.yunan.dto.Users;
import com.yunan.mapper.AuthMapper;
import com.yunan.response.ErrorResponse;
import com.yunan.service.AuthService;
import com.yunan.util.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;


import javax.annotation.Resource;
import java.util.Date;
import java.util.Random;


@Slf4j
@Service
public class AuthServiceImpl implements AuthService {
    @Resource
    private AuthMapper authMapper;
    @Autowired
    RedisUtils redisUtils;

    // 手机号正则表达式
    private static final String PHONE_REGEX = "^1[3-9]\\d{9}$";
    // 邮箱正则表达式
    private static final String EMAIL_REGEX =
            "^[a-zA-Z0-9_+&*-]+(?:\\.[a-zA-Z0-9_+&*-]+)*@(?:[a-zA-Z0-9-]+\\.)+[a-zA-Z]{2,7}$";

    public static boolean isValidPhone(String phone) {
        return phone != null && phone.matches(PHONE_REGEX);
    }

    public static boolean isValidEmail(String input) {
        return input != null && input.matches(EMAIL_REGEX);
    }

    @Override
    public ResponseEntity<Object> register(RegisterDTO registerDTO) {
        //获取正确的验证码
        Object rightCode = redisUtils.hGet("rightCode", registerDTO.getEmail());
        //处理registerDTO对数据为空的逻辑
        // 验证验证码的正确性
        if(registerDTO.getCode().equals(rightCode)){
            LambdaQueryWrapper<Users> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Users::getEmail, registerDTO.getEmail());
            if(authMapper.selectOne(queryWrapper) != null) {
                return ResponseEntity
                        .status(HttpStatus.CONFLICT)
                        .body(new ErrorResponse("用户已存在！！！"));
            }
            //为用户创建默认用户名
            String userName = createUserName();
            Users user = new Users();
            //设置账号创建时间
            Date currentDate = new Date();
            user.setCreatedTime(currentDate);
            user.setUsername(userName);
            user.setPassword(registerDTO.getPassword());
            user.setEmail(registerDTO.getEmail());
            //状态设置 " 1 " 正常账号
            user.setState(1);
            boolean isSaved = authMapper.insert(user) > 0;
            //测试
//            boolean isSaved = false;
//            boolean isSaved = true;


            if (isSaved) {
                return ResponseEntity
                        .status(HttpStatus.OK)
                        .body("注册成功！");
            } else {
                return ResponseEntity
                        .status(HttpStatus.INTERNAL_SERVER_ERROR)
                        .body("数据库存储失败，请稍后重试");
            }
        } else{
            // 如果验证码为空或者与正确的验证码不匹配，返回错误信息
            return ResponseEntity
                    .status(HttpStatus.BAD_REQUEST)  // 设置 HTTP 状态码为 400
                    .body(new ErrorResponse("验证码不正确或已过期"));  // 返回自定义的错误消息
        }

    }

    @Override
    public ResponseEntity<Object> login(LoginDTO loginDTO) {
        log.info("emai: {}", isValidEmail(loginDTO.getEmailOrPhoneNumber()));
        log.info("phone: {}", isValidPhone(loginDTO.getEmailOrPhoneNumber()));
        if(isValidPhone(loginDTO.getEmailOrPhoneNumber())) {
            loginDTO.setPhoneNumber(loginDTO.getEmailOrPhoneNumber());
        }else if(isValidEmail(loginDTO.getEmailOrPhoneNumber())) {
            loginDTO.setEmail(loginDTO.getEmailOrPhoneNumber());
        }else{
            ResponseEntity
                    .status(HttpStatus.BAD_REQUEST)
                    .body("账号校验失败");
        }
        if(loginDTO.getEmail() != null){
            LambdaQueryWrapper<Users> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Users::getEmail, loginDTO.getEmail());
            Users users = authMapper.selectOne(queryWrapper);
            if(users == null){
                return ResponseEntity
                        .status(HttpStatus.BAD_REQUEST)
                        .body(new ErrorResponse("邮箱用户不存在！！！"));
            }
            String rightPassword = users.getPassword();
            if(rightPassword.equals(loginDTO.getPassword())){
                return ResponseEntity
                        .status(HttpStatus.OK)
                        .body("登入成功！！！");
            }else {
                return ResponseEntity
                        .status(HttpStatus.BAD_REQUEST)
                        .body("邮箱或密码错误!!!");
            }

        }else if(loginDTO.getPhoneNumber() != null){
            LambdaQueryWrapper<Users> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Users::getPhoneNumber, loginDTO.getPhoneNumber());
            Users users = authMapper.selectOne(queryWrapper);
            if(users == null){
                return ResponseEntity
                        .status(HttpStatus.BAD_REQUEST)
                        .body(new ErrorResponse("手机号用户不存在！！"));
            }
            String rightPassword = users.getPassword();
            if(rightPassword.equals(loginDTO.getPassword())){
                return ResponseEntity
                        .status(HttpStatus.OK)
                        .body("登入成功！！！");
            }else {
                return ResponseEntity
                        .status(HttpStatus.BAD_REQUEST)
                        .body("手机号或密码错误!!!");
            }

        }else{
            return ResponseEntity
                    .status(HttpStatus.BAD_REQUEST)
                    .body(new ErrorResponse("其他错误待管理员处理！！！"));
        }
    }

    //随机生成用户名算法
    public String createUserName() {
        // 字符集，包括小写字母和数字
        String chars = "abcdefghijklmnopqrstuvwxyz0123456789";
        StringBuilder randomPart = new StringBuilder();

        // 随机生成6位字符
        Random random = new Random();
        for (int i = 0; i < 6; i++) {
            randomPart.append(chars.charAt(random.nextInt(chars.length())));
        }

        // 获取当前时间戳（秒级）
        long timestamp = System.currentTimeMillis() / 1000;

        // 拼接生成用户名，例如 "user_xxx123_1638326400"
        return "user_" + randomPart.toString() + "_" + timestamp;
    }

}
